/*
 * CRC_Cal.c
 *
 *  Created on: Sep 13, 2019
 *      Author: a0222913
 */
#include "BeaconTorch.h"

#define polynomialINV 0x8C //as UART transmit from LSB to MSB, invert the polynomial 0x31 (0011 0001) to 0x8C (1000 1100)
#define LSB 0x01

extern char crcArray[256];

void crcInitial() //calculate and store the CRC of data from 0x00 to 0xFF
{
    unsigned int k, j;
    char remainder;

    for(k=0;k<256;k++)
    {
        remainder=k;
        for(j=8;j>0;j--)
        {
            if(remainder & LSB)
            {
                remainder = (remainder>>1) ^ polynomialINV; //right shift 1 bit and do the XOR operation with 0x8C
            }
            else
            {
                remainder = remainder>>1;
            }
        }
        crcArray[k]=remainder;
    }
}

char CRC_LUT(char commandFrame_withoutCRC[], unsigned int byteLength) //calculate CRC of command frame
{
    unsigned int k;
    char remainder, tempData;

    remainder = 0xFF; //assign the initial value 0xFF

    for(k=0;k<byteLength;k++)
    {
        tempData = remainder ^ commandFrame_withoutCRC[k]; //input data byte XOR remainder
        remainder = crcArray[tempData]; //use tempData as the index to retrieve its CRC from crcArray
    }

    //we have to reverse the final remainder to get the CRC value, for example, if final remainder = 0010 1100, we need to reverse it to 0011 0100
    remainder = ((remainder & 0x80)>>7) + ((remainder & 0x40) >>5) + ((remainder & 0x20) >>3) + ((remainder & 0x10)>>1) + ((remainder & 0x08)<<1) + ((remainder & 0x04) <<3) + ((remainder & 0x02) <<5) + ((remainder & 0x01)<<7);
    return remainder;
}
