/*
 * FlexWire.c
 *
 *  Created on: Sep 18, 2019
 *      Author: a0222913
 */
#include "derivative.h" /* include peripheral declarations SSKEAZ64M4 */
#include "BeaconTorch.h"

void FlexBroadcast(char registerAddr, char DATA_BYTES[], char DATA_LENGTH_x)
{
    char DEV_ADDR = 0x00, REG_ADDR;
    char commandFrame[12];// one longest command frame length is 12
    char dataLength, frameLength;
    unsigned int i;

    DEV_ADDR = (0xC0 | DATA_LENGTH_x | DEV_ADDR_0);
    REG_ADDR = registerAddr;

    commandFrame[0] = 0x55; //SYNC byte
    commandFrame[1] = DEV_ADDR;
    commandFrame[2] = REG_ADDR;

    switch (DATA_LENGTH_x)
    {
    case 0:
        {
            dataLength = 1;
            break;
        }
    case 16:
        {
            dataLength = 2;
            break;
        }
    case 32:
        {
            dataLength = 4;
            break;
        }
    case 48:
        {
            dataLength = 8;
            break;
        }
    default : break;
    }

    for(i=0;i<dataLength;i++)
    {
        commandFrame[i+3] = DATA_BYTES[i];
    }

    commandFrame[i+3] = CRC_LUT(commandFrame+1,i+2); //calculate CRC of all the command frame bytes

    frameLength = i+4; //store the entire command frame byte length

    uartWrite(commandFrame, frameLength, FALSE, 0); //data will be stored in dmaRxData[]
}


void FlexWrite(char DEV_ADDR_x, char registerAddr, char DATA_BYTES[], char DATA_LENGTH_x, unsigned int checkResponse)
{
    char DEV_ADDR=0x00, REG_ADDR;
    char commandFrame[12];// one longest command frame length is 12
    char dataLength, frameLength, responseLength;
    unsigned int i;

    DEV_ADDR = (FLEX_Write | DATA_LENGTH_x | DEV_ADDR_x);
    REG_ADDR = registerAddr;

    commandFrame[0] = 0x55; //SYNC byte
    commandFrame[1] = DEV_ADDR;
    commandFrame[2] = REG_ADDR;

    switch (DATA_LENGTH_x)
    {
    case 0:
        {
            dataLength = 1;
            break;
        }
    case 16:
        {
            dataLength = 2;
            break;
        }
    case 32:
        {
            dataLength = 4;
            break;
        }
    case 48:
        {
            dataLength = 8;
            break;
        }
    default : break;
    }

    for(i=0;i<dataLength;i++)
    {
        commandFrame[i+3] = DATA_BYTES[i];
    }

    commandFrame[i+3] = CRC_LUT(commandFrame+1,i+2); //calculate CRC of all the command frame bytes

    frameLength = i+4; //store the entire command frame byte length

    //~~~~~~~~~CAN is not used~~~~~~~~~~~//
    //responseLength = 2; //as CAN Transceiver is connected, all the sent data will also loop back to receive terminal of MCU
                                    //so, the number of response number should be (Num of sent data + Num of real response data + CRC)
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//

    //~~~~~~~~~~CAN is used~~~~~~~~~~~~~//
    responseLength=frameLength+2;
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//

    uartWrite(commandFrame, frameLength, checkResponse, responseLength); //data will be stored in dmaRxData[]

    // if checkRepsonse is not set, a minimum wait time is needed before writing next command frame, to make sure next writing command is after response finished sending
    // the minimum wait time can be calculated by the formula below
    // minimum wait time = response length * 10 *1/baurdrate

}

void FlexRead(char DEV_ADDR_x, char registerAddr, char DATA_LENGTH_x)
{
    char DEV_ADDR=0x00, REG_ADDR;
    char commandFrame[4];
    unsigned int dataLength, responseLength;
    DEV_ADDR = (FLEX_Read | DATA_LENGTH_x | DEV_ADDR_x);
    REG_ADDR = registerAddr;

    commandFrame[0] = 0x55; //SYNC byte
    commandFrame[1] = DEV_ADDR;
    commandFrame[2] = REG_ADDR;

    switch (DATA_LENGTH_x)
    {
    case 0:
        {
            dataLength = 1;
            break;
        }
    case 16:
        {
            dataLength = 2;
            break;
        }
    case 32:
        {
            dataLength = 4;
            break;
        }
    case 48:
        {
            dataLength = 8;
            break;
        }
    default : break;
    }

    commandFrame[3] = CRC_LUT(commandFrame+1,2); //calculate CRC of all the command frame bytes
    //~~~~~~~CAN is not used~~~~~~~~~~~//
    //responseLength = dataLength+1;//as CAN Transceiver is connected, all the sent data will also loop back to receive terminal of MCU
    //so, the number of response number should be (Num of sent data + Num of real response data + CRC)
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~//

    //~~~~~~CAN is used~~~~~~~~~~~~~~~~//
    responseLength = 4+dataLength+1;
    //~~~~~~~~~~~~~~~~~~~~~~//
    uartWrite(commandFrame, 4, TRUE, responseLength); //data will be stored in dmaRxData[]

}
